@@ -1,123 +1,99 @@
 package Moo::Role;
-use strict;
-use warnings;
 
-use Moo::_Utils qw(
-  _check_tracked
-  _getglob
-  _getstash
-  _install_coderef
-  _install_modifier
-  _install_tracked
-  _load_module
-  _name_coderef
-  _set_loaded
-  _unimport_coderefs
-);
-use Carp qw(croak);
+use Moo::_strictures;
+use Moo::_Utils;
 use Role::Tiny ();
-BEGIN { our @ISA = qw(Role::Tiny) }
-BEGIN {
-  our @CARP_NOT = qw(
-    Method::Generate::Accessor
-    Method::Generate::Constructor
-    Moo::sification
-    Moo::_Utils
-    Role::Tiny
-  );
-}
+our @ISA = qw(Role::Tiny);
 
-our $VERSION = '2.005_002';
-$VERSION =~ tr/_//d;
+our $VERSION = '2.000001';
+$VERSION = eval $VERSION;
 
 require Moo::sification;
-Moo::sification->import;
 
 BEGIN {
     *INFO = \%Role::Tiny::INFO;
     *APPLIED_TO = \%Role::Tiny::APPLIED_TO;
-    *COMPOSED = \%Role::Tiny::COMPOSED;
     *ON_ROLE_CREATE = \@Role::Tiny::ON_ROLE_CREATE;
 }
 
 our %INFO;
 our %APPLIED_TO;
 our %APPLY_DEFAULTS;
-our %COMPOSED;
 our @ON_ROLE_CREATE;
 
+sub _install_tracked {
+  my ($target, $name, $code) = @_;
+  $INFO{$target}{exports}{$name} = $code;
+  _install_coderef "${target}::${name}" => "Moo::Role::${name}" => $code;
+}
+
 sub import {
   my $target = caller;
-  if ($Moo::MAKERS{$target} and $Moo::MAKERS{$target}{is_class}) {
-    croak "Cannot import Moo::Role into a Moo class";
-  }
+  my ($me) = @_;
+
   _set_loaded(caller);
-  goto &Role::Tiny::import;
-}
-
-sub _accessor_maker_for {
-  my ($class, $target) = @_;
-  ($INFO{$target}{accessor_maker} ||= do {
-    require Method::Generate::Accessor;
-    Method::Generate::Accessor->new
-  });
-}
-
-sub _install_subs {
-  my ($me, $target) = @_;
-  my %install = $me->_gen_subs($target);
-  _install_tracked $target => $_ => $install{$_}
-    for sort keys %install;
+  strict->import;
+  warnings->import;
+  if ($Moo::MAKERS{$target} and $Moo::MAKERS{$target}{is_class}) {
+    die "Cannot import Moo::Role into a Moo class";
+  }
+  $INFO{$target} ||= {};
+  # get symbol table reference
+  my $stash = _getstash($target);
+  _install_tracked $target => has => sub {
+    my $name_proto = shift;
+    my @name_proto = ref $name_proto eq 'ARRAY' ? @$name_proto : $name_proto;
+    if (@_ % 2 != 0) {
+      require Carp;
+      Carp::croak("Invalid options for " . join(', ', map "'$_'", @name_proto)
+        . " attribute(s): even number of arguments expected, got " . scalar @_)
+    }
+    my %spec = @_;
+    foreach my $name (@name_proto) {
+      my $spec_ref = @name_proto > 1 ? +{%spec} : \%spec;
+      ($INFO{$target}{accessor_maker} ||= do {
+        require Method::Generate::Accessor;
+        Method::Generate::Accessor->new
+      })->generate_method($target, $name, $spec_ref);
+      push @{$INFO{$target}{attributes}||=[]}, $name, $spec_ref;
+      $me->_maybe_reset_handlemoose($target);
+    }
+  };
+  # install before/after/around subs
+  foreach my $type (qw(before after around)) {
+    _install_tracked $target => $type => sub {
+      require Class::Method::Modifiers;
+      push @{$INFO{$target}{modifiers}||=[]}, [ $type => @_ ];
+      $me->_maybe_reset_handlemoose($target);
+    };
+  }
+  _install_tracked $target => requires => sub {
+    push @{$INFO{$target}{requires}||=[]}, @_;
+    $me->_maybe_reset_handlemoose($target);
+  };
+  _install_tracked $target => with => sub {
+    $me->apply_roles_to_package($target, @_);
+    $me->_maybe_reset_handlemoose($target);
+  };
+  return if $me->is_role($target); # already exported into this package
+  $INFO{$target}{is_role} = 1;
   *{_getglob("${target}::meta")} = $me->can('meta');
-  return;
-}
+  # grab all *non-constant* (stash slot is not a scalarref) subs present
+  # in the symbol table and store their refaddrs (no need to forcibly
+  # inflate constant subs into real subs) - also add '' to here (this
+  # is used later) with a map to the coderefs in case of copying or re-use
+  my @not_methods = ('', map { *$_{CODE}||() } grep !ref($_), values %$stash);
+  @{$INFO{$target}{not_methods}={}}{@not_methods} = @not_methods;
+  # a role does itself
+  $APPLIED_TO{$target} = { $target => undef };
 
-sub _require_module {
-  _load_module($_[1]);
-}
-
-sub _gen_subs {
-  my ($me, $target) = @_;
-  return (
-    has => sub {
-      my $name_proto = shift;
-      my @name_proto = ref $name_proto eq 'ARRAY' ? @$name_proto : $name_proto;
-      if (@_ % 2 != 0) {
-        croak("Invalid options for " . join(', ', map "'$_'", @name_proto)
-          . " attribute(s): even number of arguments expected, got " . scalar @_)
-      }
-      my %spec = @_;
-      foreach my $name (@name_proto) {
-        my $spec_ref = @name_proto > 1 ? +{%spec} : \%spec;
-        $me->_accessor_maker_for($target)
-          ->generate_method($target, $name, $spec_ref);
-        push @{$INFO{$target}{attributes}||=[]}, $name, $spec_ref;
-        $me->_maybe_reset_handlemoose($target);
-      }
-    },
-    (map {
-      my $type = $_;
-      (
-        $type => sub {
-          push @{$INFO{$target}{modifiers}||=[]}, [ $type => @_ ];
-          $me->_maybe_reset_handlemoose($target);
-        },
-      )
-    } qw(before after around)),
-    requires => sub {
-      push @{$INFO{$target}{requires}||=[]}, @_;
-      $me->_maybe_reset_handlemoose($target);
-    },
-    with => sub {
-      $me->apply_roles_to_package($target, @_);
-      $me->_maybe_reset_handlemoose($target);
-    },
-  );
+  $_->($target)
+    for @ON_ROLE_CREATE;
 }
 
 push @ON_ROLE_CREATE, sub {
   my $target = shift;
-  if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
+  if ($INC{'Moo/HandleMoose.pm'}) {
     Moo::HandleMoose::inject_fake_metaclass_for($target);
   }
 };
@@ -131,27 +107,22 @@ sub meta {
 
 sub unimport {
   my $target = caller;
-  _unimport_coderefs($target);
+  _unimport_coderefs($target, $INFO{$target});
 }
 
 sub _maybe_reset_handlemoose {
   my ($class, $target) = @_;
-  if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
+  if ($INC{"Moo/HandleMoose.pm"}) {
     Moo::HandleMoose::maybe_reinject_fake_metaclass_for($target);
   }
 }
 
-sub _non_methods {
-  my $self = shift;
-  my ($role) = @_;
-
-  my $non_methods = $self->SUPER::_non_methods(@_);
-
-  my $all_subs = $self->_all_subs($role);
-  $non_methods->{$_} = $all_subs->{$_}
-    for _check_tracked($role, [ keys %$all_subs ]);
-
-  return $non_methods;
+sub methods_provided_by {
+  my ($self, $role) = @_;
+  _load_module($role);
+  $self->_inhale_if_moose($role);
+  die "${role} is not a Moo::Role" unless $self->is_role($role);
+  return $self->SUPER::methods_provided_by($role);
 }
 
 sub is_role {
@@ -194,7 +165,6 @@ sub _inhale_if_moose {
         my $spec = { %{ $is_mouse ? $attr : $attr->original_options } };
 
         if ($spec->{isa}) {
-          require Sub::Quote;
 
           my $get_constraint = do {
             my $pkg = $is_mouse
@@ -206,17 +176,10 @@ sub _inhale_if_moose {
 
           my $tc = $get_constraint->($spec->{isa});
           my $check = $tc->_compiled_type_constraint;
-          my $tc_var = '$_check_for_'.Sub::Quote::sanitize_identifier($tc->name);
 
-          $spec->{isa} = Sub::Quote::quote_sub(
-            qq{
-              &${tc_var} or Carp::croak "Type constraint failed for \$_[0]"
-            },
-            { $tc_var => \$check },
-            {
-              package => $role,
-            },
-          );
+          $spec->{isa} = sub {
+            &$check or die "Type constraint failed for $_[0]"
+          };
 
           if ($spec->{coerce}) {
 
@@ -243,6 +206,7 @@ sub _inhale_if_moose {
         }
       }
     }
+    require Class::Method::Modifiers if @$mods;
     $INFO{$role}{inhaled_from_moose} = 1;
     $INFO{$role}{is_role} = 1;
   }
@@ -283,102 +247,117 @@ sub _make_accessors {
   }
 }
 
-sub _undefer_subs {
-  my ($self, $target, $role) = @_;
-  if ($INC{'Sub/Defer.pm'}) {
-    Sub::Defer::undefer_package($role);
-  }
-}
-
 sub role_application_steps {
-  qw(_handle_constructor _undefer_subs _maybe_make_accessors),
+  qw(_handle_constructor _maybe_make_accessors),
     $_[0]->SUPER::role_application_steps;
 }
 
-sub _build_class_with_roles {
-  my ($me, $new_name, $superclass, @roles) = @_;
-  $Moo::MAKERS{$new_name} = {is_class => 1};
-  $me->SUPER::_build_class_with_roles($new_name, $superclass, @roles);
-
-  if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
-    Moo::HandleMoose::inject_fake_metaclass_for($new_name);
+sub apply_roles_to_package {
+  my ($me, $to, @roles) = @_;
+  foreach my $role (@roles) {
+    _load_module($role);
+    $me->_inhale_if_moose($role);
+    die "${role} is not a Moo::Role" unless $me->is_role($role);
   }
-  _set_loaded($new_name, (caller)[1]);
-  return $new_name;
+  $me->SUPER::apply_roles_to_package($to, @roles);
 }
 
-sub _gen_apply_defaults_for {
-  my ($me, $class, @roles) = @_;
+sub apply_single_role_to_package {
+  my ($me, $to, $role) = @_;
+  _load_module($role);
+  $me->_inhale_if_moose($role);
+  die "${role} is not a Moo::Role" unless $me->is_role($role);
+  $me->SUPER::apply_single_role_to_package($to, $role);
+}
 
-  my @attrs = map @{$INFO{$_}{attributes}||[]}, @roles;
+sub create_class_with_roles {
+  my ($me, $superclass, @roles) = @_;
 
-  my $con_gen;
-  my $m;
+  my ($new_name, $compose_name) = $me->_composite_name($superclass, @roles);
 
-  return undef
-    unless $INC{'Moo.pm'}
-    and @attrs
-    and $con_gen = Moo->_constructor_maker_for($class)
-    and $m = Moo->_accessor_maker_for($class);
+  return $new_name if $Role::Tiny::COMPOSED{class}{$new_name};
 
-  my $specs = $con_gen->all_attribute_specs;
-
-  my %seen;
-  my %captures;
-  my @set;
-  while (my ($name, $spec) = splice @attrs, 0, 2) {
-    next
-      if $seen{$name}++;
-
-    next
-      unless $m->has_eager_default($name, $spec);
-
-    my ($has, $has_cap)
-      = $m->generate_simple_has('$_[0]', $name, $spec);
-    my ($set, $pop_cap)
-      = $m->generate_use_default('$_[0]', $name, $spec, $has);
-
-    @captures{keys %$has_cap, keys %$pop_cap}
-      = (values %$has_cap, values %$pop_cap);
-
-    push @set, $set;
+  foreach my $role (@roles) {
+      _load_module($role);
+      $me->_inhale_if_moose($role);
   }
 
-  return undef
-    if !@set;
+  my $m;
+  if ($INC{"Moo.pm"}
+      and $m = Moo->_accessor_maker_for($superclass)
+      and ref($m) ne 'Method::Generate::Accessor') {
+    # old fashioned way time.
+    *{_getglob("${new_name}::ISA")} = [ $superclass ];
+    $Moo::MAKERS{$new_name} = {is_class => 1};
+    $me->apply_roles_to_package($new_name, @roles);
+    _set_loaded($new_name, (caller)[1]);
+    return $new_name;
+  }
 
-  my $code = join '', map "($_),", @set;
-  no warnings 'void';
-  require Sub::Quote;
-  return Sub::Quote::quote_sub(
-    "${class}::_apply_defaults",
-    $code,
-    \%captures,
-    {
-      package => $class,
-      no_install => 1,
-      no_defer => 1,
-    }
-  );
+  $me->SUPER::create_class_with_roles($superclass, @roles);
+
+  foreach my $role (@roles) {
+    die "${role} is not a Moo::Role" unless $me->is_role($role);
+  }
+
+  $Moo::MAKERS{$new_name} = {is_class => 1};
+
+  $me->_handle_constructor($new_name, $_) for @roles;
+
+  _set_loaded($new_name, (caller)[1]);
+  return $new_name;
 }
 
 sub apply_roles_to_object {
   my ($me, $object, @roles) = @_;
   my $new = $me->SUPER::apply_roles_to_object($object, @roles);
-  my $class = ref $new;
-  _set_loaded($class, (caller)[1]);
+  _set_loaded(ref $new, (caller)[1]);
 
-  if (!exists $APPLY_DEFAULTS{$class}) {
-    $APPLY_DEFAULTS{$class} = $me->_gen_apply_defaults_for($class, @roles);
-  }
-  if (my $apply_defaults = $APPLY_DEFAULTS{$class}) {
-    local $Carp::Internal{+__PACKAGE__} = 1;
-    local $Carp::Internal{$class} = 1;
-    $new->$apply_defaults;
-  }
+  my $apply_defaults = $APPLY_DEFAULTS{ref $new} ||= do {
+    my %attrs = map { @{$INFO{$_}{attributes}||[]} } @roles;
+
+    if ($INC{'Moo.pm'}
+        and keys %attrs
+        and my $con_gen = Moo->_constructor_maker_for(ref $new)
+        and my $m = Moo->_accessor_maker_for(ref $new)) {
+      require Sub::Quote;
+
+      my $specs = $con_gen->all_attribute_specs;
+
+      my $assign = "{no warnings 'void';\n";
+      my %captures;
+      foreach my $name ( keys %attrs ) {
+        my $spec = $specs->{$name};
+        if ($m->has_eager_default($name, $spec)) {
+          my ($has, $has_cap)
+            = $m->generate_simple_has('$_[0]', $name, $spec);
+          my ($code, $pop_cap)
+            = $m->generate_use_default('$_[0]', $name, $spec, $has);
+
+          $assign .= $code . ";\n";
+          @captures{keys %$has_cap, keys %$pop_cap}
+            = (values %$has_cap, values %$pop_cap);
+        }
+      }
+      $assign .= "}";
+      Sub::Quote::quote_sub($assign, \%captures);
+    }
+    else {
+      sub {};
+    }
+  };
+  $new->$apply_defaults;
   return $new;
 }
 
+sub _composable_package_for {
+  my ($self, $role) = @_;
+  my $composed_name = 'Role::Tiny::_COMPOSABLE::'.$role;
+  return $composed_name if $Role::Tiny::COMPOSED{role}{$composed_name};
+  $self->_make_accessors_if_moose($composed_name, $role);
+  $self->SUPER::_composable_package_for($role);
+}
+
 sub _install_single_modifier {
   my ($me, @args) = @_;
   _install_modifier(@args);
@@ -400,7 +379,6 @@ sub does_role {
   if ($INC{'Moose.pm'}
       and $meta = Class::MOP::class_of($proto)
       and ref $meta ne 'Moo::HandleMoose::FakeMetaClass'
-      and $meta->can('does_role')
   ) {
     return $meta->does_role($role);
   }
@@ -441,41 +419,40 @@ Moo::Role - Minimal Object Orientation support for Roles
 
 =head1 SYNOPSIS
 
-  package My::Role;
+ package My::Role;
 
-  use Moo::Role;
-  use strictures 2;
+ use Moo::Role;
+ use strictures 2;
 
-  sub foo { ... }
+ sub foo { ... }
 
-  sub bar { ... }
+ sub bar { ... }
 
-  has baz => (
-    is => 'ro',
-  );
+ has baz => (
+   is => 'ro',
+ );
 
-  1;
+ 1;
 
 And elsewhere:
 
-  package Some::Class;
+ package Some::Class;
 
-  use Moo;
-  use strictures 2;
+ use Moo;
+ use strictures 2;
 
-  # bar gets imported, but not foo
-  with 'My::Role';
+ # bar gets imported, but not foo
+ with('My::Role');
 
-  sub foo { ... }
+ sub foo { ... }
 
-  1;
+ 1;
 
 =head1 DESCRIPTION
 
 C<Moo::Role> builds upon L<Role::Tiny>, so look there for most of the
-documentation on how this works (in particular, using C<Moo::Role> also
-enables L<strict> and L<warnings>).  The main addition here is extra bits to
-make the roles more "Moosey;" which is to say, it adds L</has>.
+documentation on how this works.  The main addition here is extra bits to make
+the roles more "Moosey;" which is to say, it adds L</has>.
 
 =head1 IMPORTED SUBROUTINES
 
@@ -484,9 +461,9 @@ imported by this module.
 
 =head2 has
 
-  has attr => (
-    is => 'ro',
-  );
+ has attr => (
+   is => 'ro',
+ );
 
 Declares an attribute for the class to be composed into.  See
 L<Moo/has> for all options.
@@ -498,18 +475,18 @@ declared before the C<use Moo::Role> statement automatically.
 Anything imported after C<use Moo::Role> will be composed into
 consuming packages.  A package that consumes this role:
 
-  package My::Role::ID;
+ package My::Role::ID;
 
-  use Digest::MD5 qw(md5_hex);
-  use Moo::Role;
-  use Digest::SHA qw(sha1_hex);
+ use Digest::MD5 qw(md5_hex);
+ use Moo::Role;
+ use Digest::SHA qw(sha1_hex);
 
-  requires 'name';
+ requires 'name';
 
-  sub as_md5  { my ($self) = @_; return md5_hex($self->name);  }
-  sub as_sha1 { my ($self) = @_; return sha1_hex($self->name); }
+ sub as_md5  { my ($self) = @_; return md5_hex($self->name);  }
+ sub as_sha1 { my ($self) = @_; return sha1_hex($self->name); }
 
-  1;
+ 1;
 
 ..will now have a C<< $self->sha1_hex() >> method available to it
 that probably does not do what you expect.  On the other hand, a call
